@@ -38,6 +38,7 @@
 #include <haproxy/ticks.h>
 
 #include <haproxy/listener.h>
+#include <haproxy/proto_quic.h>
 #include <haproxy/quic_cc.h>
 #include <haproxy/quic_conn-t.h>
 #include <haproxy/quic_enc.h>
@@ -120,42 +121,6 @@ static inline size_t quic_saddr_cpy(unsigned char *buf,
 	return p - buf;
 }
 
-/* Concatenate the port and address of <saddr> to <cid> QUIC connection ID. The
- * <addrlen> field of <cid> will be updated with the size of the concatenated
- * address.
- *
- * Returns the number of bytes concatenated to <cid>.
- */
-static inline size_t quic_cid_saddr_cat(struct quic_cid *cid,
-                                        struct sockaddr_storage *saddr)
-{
-	void *port, *addr;
-	size_t port_len, addr_len;
-
-	cid->addrlen = 0;
-
-	if (saddr->ss_family == AF_INET6) {
-		port = &((struct sockaddr_in6 *)saddr)->sin6_port;
-		addr = &((struct sockaddr_in6 *)saddr)->sin6_addr;
-		port_len = sizeof ((struct sockaddr_in6 *)saddr)->sin6_port;
-		addr_len = sizeof ((struct sockaddr_in6 *)saddr)->sin6_addr;
-	}
-	else {
-		port = &((struct sockaddr_in *)saddr)->sin_port;
-		addr = &((struct sockaddr_in *)saddr)->sin_addr;
-		port_len = sizeof ((struct sockaddr_in *)saddr)->sin_port;
-		addr_len = sizeof ((struct sockaddr_in *)saddr)->sin_addr;
-	}
-
-	memcpy(cid->data + cid->len, port, port_len);
-	cid->addrlen += port_len;
-	memcpy(cid->data + cid->len + port_len, addr, addr_len);
-	cid->addrlen += addr_len;
-
-	return port_len + addr_len;
-}
-
-
 /* Dump the QUIC connection ID value if present (non null length). Used only for
  * debugging purposes.
  * Always succeeds.
@@ -173,26 +138,58 @@ static inline void quic_cid_dump(struct buffer *buf,
 	chunk_appendf(buf, ")");
 }
 
-/* Free the CIDs attached to <conn> QUIC connection. This must be called under
- * the CID lock.
- */
+/* Return tree index where <cid> is stored. */
+static inline uchar _quic_cid_tree_idx(const unsigned char *cid)
+{
+	return cid[0];
+}
+
+/* Return tree index where <cid> is stored. */
+static inline uchar quic_cid_tree_idx(const struct quic_cid *cid)
+{
+	return _quic_cid_tree_idx(cid->data);
+}
+
+/* Insert <conn_id> into global CID tree as a thread-safe operation. */
+static inline void quic_cid_insert(struct quic_connection_id *conn_id)
+{
+	const uchar idx = quic_cid_tree_idx(&conn_id->cid);
+	struct quic_cid_tree *tree = &quic_cid_trees[idx];
+
+	HA_RWLOCK_WRLOCK(QC_CID_LOCK, &tree->lock);
+	ebmb_insert(&tree->root, &conn_id->node, conn_id->cid.len);
+	HA_RWLOCK_WRUNLOCK(QC_CID_LOCK, &tree->lock);
+}
+
+/* Remove <conn_id> from global CID tree as a thread-safe operation. */
+static inline void quic_cid_delete(struct quic_connection_id *conn_id)
+{
+	const uchar idx = quic_cid_tree_idx(&conn_id->cid);
+	struct quic_cid_tree __maybe_unused *tree = &quic_cid_trees[idx];
+
+	HA_RWLOCK_WRLOCK(QC_CID_LOCK, &tree->lock);
+	ebmb_delete(&conn_id->node);
+	HA_RWLOCK_WRUNLOCK(QC_CID_LOCK, &tree->lock);
+}
+
+/* Free the CIDs attached to <conn> QUIC connection. */
 static inline void free_quic_conn_cids(struct quic_conn *conn)
 {
 	struct eb64_node *node;
 
 	node = eb64_first(&conn->cids);
 	while (node) {
-		struct quic_connection_id *cid;
+		struct quic_connection_id *conn_id;
 
-		cid = eb64_entry(node, struct quic_connection_id, seq_num);
+		conn_id = eb64_entry(node, struct quic_connection_id, seq_num);
 
 		/* remove the CID from the receiver tree */
-		ebmb_delete(&cid->node);
+		quic_cid_delete(conn_id);
 
 		/* remove the CID from the quic_conn tree */
 		node = eb64_next(node);
-		eb64_delete(&cid->seq_num);
-		pool_free(pool_head_quic_connection_id, cid);
+		eb64_delete(&conn_id->seq_num);
+		pool_free(pool_head_quic_connection_id, conn_id);
 	}
 }
 
@@ -202,57 +199,13 @@ static inline void free_quic_conn_cids(struct quic_conn *conn)
 static inline void quic_connection_id_to_frm_cpy(struct quic_frame *dst,
                                                  struct quic_connection_id *src)
 {
-	struct quic_new_connection_id *to = &dst->new_connection_id;
+	struct qf_new_connection_id *ncid_frm = &dst->new_connection_id;
 
-	to->seq_num = src->seq_num.key;
-	to->retire_prior_to = src->retire_prior_to;
-	to->cid.len = src->cid.len;
-	to->cid.data = src->cid.data;
-	to->stateless_reset_token = src->stateless_reset_token;
-}
-
-/* extract a TID from a CID for receiver <rx>, from 0 to global.nbthread-1 and
- * in any case no more than 4095. It takes into account the bind_conf's thread
- * group and the bind_conf's thread mask. The algorithm is the following: most
- * packets contain a valid thread ID for the bind_conf, which means that the
- * retrieved ID directly maps to a bound thread ID. If that's not the case,
- * then we have to remap it. The resulting thread ID will then differ but will
- * be correctly encoded and decoded.
- */
-static inline uint quic_get_cid_tid(const unsigned char *cid, const struct receiver *rx)
-{
-	uint id, grp;
-	uint base, count;
-
-	id = read_n16(cid) & 4095;
-	grp = rx->bind_tgroup;
-	base  = ha_tgroup_info[grp - 1].base;
-	count = ha_tgroup_info[grp - 1].count;
-
-	if (base <= id && id < base + count &&
-	    rx->bind_thread & ha_thread_info[id].ltid_bit)
-		return id; // part of the group and bound: valid
-
-	/* The thread number isn't valid, it doesn't map to a thread bound on
-	 * this receiver. Let's reduce it to one of the thread(s) valid for
-	 * that receiver.
-	 */
-	count = my_popcountl(rx->bind_thread);
-	id = count - 1 - id % count;
-	id = mask_find_rank_bit(id, rx->bind_thread);
-	id += base;
-	return id;
-}
-
-/* Modify <cid> to have a CID linked to the thread ID <target_tid> that
- * quic_get_cid_tid() will be able to extract return.
- */
-static inline void quic_pin_cid_to_tid(unsigned char *cid, uint target_tid)
-{
-	uint16_t prev_id;
-
-	prev_id = read_n16(cid);
-	write_n16(cid, (prev_id & ~4095) | target_tid);
+	ncid_frm->seq_num = src->seq_num.key;
+	ncid_frm->retire_prior_to = src->retire_prior_to;
+	ncid_frm->cid.len = src->cid.len;
+	ncid_frm->cid.data = src->cid.data;
+	ncid_frm->stateless_reset_token = src->stateless_reset_token;
 }
 
 /* Return a 32-bits integer in <val> from QUIC packet with <buf> as address.
@@ -435,7 +388,7 @@ static inline int quic_packet_number_encode(unsigned char **buf,
  * <conn> QUIC connection. Note that the value of <ack_delay> coming from
  * ACK frame is in microseconds.
  */
-static inline unsigned int quic_ack_delay_ms(struct quic_ack *ack_frm,
+static inline unsigned int quic_ack_delay_ms(struct qf_ack *ack_frm,
                                              struct quic_conn *conn)
 {
 	return (ack_frm->ack_delay << conn->tx.params.ack_delay_exponent) / 1000;
@@ -574,6 +527,7 @@ static inline void quic_path_init(struct quic_path *path, int ipv4,
 	quic_loss_init(&path->loss);
 	path->mtu = max_dgram_sz;
 	path->cwnd = QUIC_MIN(10 * max_dgram_sz, QUIC_MAX(max_dgram_sz << 1, 14720U));
+	path->mcwnd = path->cwnd;
 	path->min_cwnd = max_dgram_sz << 1;
 	path->prep_in_flight = 0;
 	path->in_flight = 0;
@@ -709,9 +663,14 @@ int quic_set_app_ops(struct quic_conn *qc, const unsigned char *alpn, size_t alp
 int qc_check_dcid(struct quic_conn *qc, unsigned char *dcid, size_t dcid_len);
 int quic_get_dgram_dcid(unsigned char *buf, const unsigned char *end,
                         unsigned char **dcid, size_t *dcid_len);
+struct quic_cid quic_derive_cid(const struct quic_cid *orig,
+                                const struct sockaddr_storage *addr);
+int quic_get_cid_tid(const unsigned char *cid, size_t cid_len,
+                     const struct sockaddr_storage *cli_addr,
+                     unsigned char *buf, size_t buf_len);
 int qc_send_mux(struct quic_conn *qc, struct list *frms);
 
-void qc_notify_close(struct quic_conn *qc);
+void qc_notify_err(struct quic_conn *qc);
 int qc_notify_send(struct quic_conn *qc);
 
 void qc_release_frm(struct quic_conn *qc, struct quic_frame *frm);
@@ -722,6 +681,8 @@ void quic_conn_release(struct quic_conn *qc);
 
 void qc_kill_conn(struct quic_conn *qc);
 
+int qc_parse_hd_form(struct quic_rx_packet *pkt,
+                     unsigned char **buf, const unsigned char *end);
 int quic_dgram_parse(struct quic_dgram *dgram, struct quic_conn *qc,
                      struct listener *li);
 
@@ -739,5 +700,8 @@ static inline void quic_handle_stopping(void)
 	}
 }
 
+int qc_set_tid_affinity(struct quic_conn *qc, uint tid);
+void qc_finalize_affinity_rebind(struct quic_conn *qc);
+
 #endif /* USE_QUIC */
 #endif /* _HAPROXY_QUIC_CONN_H */
