@@ -175,11 +175,11 @@ void ha_backtrace_to_stderr(void)
  */
 void ha_thread_dump(struct buffer *buf, int thr, int calling_tid)
 {
-	unsigned long thr_bit = ha_thread_info[thr].ltid_bit;
+	unsigned long __maybe_unused thr_bit = ha_thread_info[thr].ltid_bit;
+	int __maybe_unused tgrp  = ha_thread_info[thr].tgid;
 	unsigned long long p = ha_thread_ctx[thr].prev_cpu_time;
 	unsigned long long n = now_cpu_time_thread(thr);
 	int stuck = !!(ha_thread_ctx[thr].flags & TH_FL_STUCK);
-	int tgrp  = ha_thread_info[thr].tgid;
 
 	chunk_appendf(buf,
 	              "%c%cThread %-2u: id=0x%llx act=%d glob=%d wq=%d rq=%d tl=%d tlsz=%d rqsz=%d\n"
@@ -198,12 +198,14 @@ void ha_thread_dump(struct buffer *buf, int thr, int calling_tid)
 	              ha_thread_ctx[thr].rq_total,
 		      ha_thread_info[thr].tgid, ha_thread_info[thr].ltid + 1,
 	              stuck,
-	              !!(th_ctx->flags & TH_FL_TASK_PROFILING));
+	              !!(ha_thread_ctx[thr].flags & TH_FL_TASK_PROFILING));
 
+#if defined(USE_THREAD)
 	chunk_appendf(buf,
-	              " harmless=%d wantrdv=%d",
+	              " harmless=%d isolated=%d",
 	              !!(_HA_ATOMIC_LOAD(&ha_tgroup_ctx[tgrp-1].threads_harmless) & thr_bit),
-	              !!(th_ctx->flags & TH_FL_TASK_PROFILING));
+		      isolated_thread == thr);
+#endif
 
 	chunk_appendf(buf, "\n");
 	chunk_appendf(buf, "             cpu_ns: poll=%llu now=%llu diff=%llu\n", p, n, n-p);
@@ -298,9 +300,10 @@ void ha_task_dump(struct buffer *buf, const struct task *task, const char *pfx)
 	if (hlua && hlua->T) {
 		chunk_appendf(buf, "stack traceback:\n    ");
 		append_prefixed_str(buf, hlua_traceback(hlua->T, "\n    "), pfx, '\n', 0);
-		b_putchr(buf, '\n');
 	}
-	else
+
+	/* we may need to terminate the current line */
+	if (*b_peek(buf, b_data(buf)-1) != '\n')
 		b_putchr(buf, '\n');
 #endif
 }
