@@ -24,7 +24,6 @@
 #include <haproxy/connection.h>
 #include <haproxy/dynbuf.h>
 #include <haproxy/fd.h>
-#include <haproxy/freq_ctr.h>
 #include <haproxy/global-t.h>
 #include <haproxy/list.h>
 #include <haproxy/listener.h>
@@ -633,17 +632,12 @@ int qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
 	} while (ret < 0 && errno == EINTR);
 
 	if (ret < 0) {
-		struct proxy *prx = qc->li->bind_conf->frontend;
-		struct quic_counters *prx_counters =
-		  EXTRA_COUNTERS_GET(prx->extra_counters_fe,
-		                     &quic_stats_module);
-
 		if (errno == EAGAIN || errno == EWOULDBLOCK ||
 		    errno == ENOTCONN || errno == EINPROGRESS) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK)
-				HA_ATOMIC_INC(&prx_counters->socket_full);
+				qc->cntrs.socket_full++;
 			else
-				HA_ATOMIC_INC(&prx_counters->sendto_err);
+				qc->cntrs.sendto_err++;
 
 			/* transient error */
 			fd_want_send(qc->fd);
@@ -654,7 +648,7 @@ int qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
 		}
 		else {
 			/* unrecoverable error */
-			HA_ATOMIC_INC(&prx_counters->sendto_err_unknown);
+			qc->cntrs.sendto_err_unknown++;
 			TRACE_PRINTF(TRACE_LEVEL_USER, QUIC_EV_CONN_SPPKTS, qc, 0, 0, 0,
 			             "UDP send failure errno=%d (%s)", errno, strerror(errno));
 			return -1;
@@ -664,13 +658,6 @@ int qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
 	if (ret != sz)
 		return 0;
 
-	/* we count the total bytes sent, and the send rate for 32-byte blocks.
-	 * The reason for the latter is that freq_ctr are limited to 4GB and
-	 * that it's not enough per second.
-	 */
-	_HA_ATOMIC_ADD(&global.out_bytes, ret);
-	update_freq_ctr(&global.out_32bps, (ret + 16) / 32);
-
 	return ret;
 }
 
@@ -763,7 +750,7 @@ int qc_rcv_buf(struct quic_conn *qc)
 
 			rxbuf_tail = (unsigned char *)b_tail(&rxbuf->buf);
 			__b_putblk(&rxbuf->buf, (char *)dgram_buf, new_dgram->len);
-			if (!quic_lstnr_dgram_dispatch(rxbuf_tail, ret, l, &qc->peer_addr, &daddr,
+			if (!quic_lstnr_dgram_dispatch(rxbuf_tail, ret, l, &saddr, &daddr,
 			                               new_dgram, &rxbuf->dgram_list)) {
 				/* TODO count lost datagrams. */
 				b_sub(&buf, ret);
